@@ -1,34 +1,29 @@
 #' Predicted line from model fit
 #'
 #' Predicted values and a confidence band are computed and, by default, plotted.
-#' \code{stat_poly_line()} behaves like \code{\link[ggplot2]{stat_smooth}}
-#' except for supporting the use of \code{y} as explanatory variable in the
-#' model formula, fits the model with \code{stats::lm()} as default for
-#' \code{method}, irrespective of the number of observations. The fit can
-#' alternatively by done by any of the methods supported by
-#' \code{\link[ggplot2]{stat_smooth}}, including \code{method = "auto"}.
 #'
 #' @details
-#' This statistic is just \code{\link[ggplot2]{stat_smooth}} with different
-#' defaults and updated so that it interprets the argument passed to
-#' \code{formula} differently, accepting \code{y} as explanatory variable and
-#' setting \code{orientation} automatically. In addition the default for
-#' \code{method} is \code{"lm"}, matching the default used in
-#' \code{stat_poly_line()}, \code{stat_quant_line()} and \code{stat_quant_eq()}.
-#' It calls \code{\link[ggplot2]{StatSmooth}} to build a layer.
+#' This statistic is similar to \code{\link[ggplot2]{stat_smooth}} but has
+#' different defaults and it interprets the argument passed to \code{formula}
+#' differently, accepting \code{y} as explanatory variable and setting
+#' \code{orientation} automatically. The default for \code{method} is
+#' \code{"lm"} and spline-based smoothers like \code{loess} are not supported.
+#' Other defaults are consistent with those in \code{stat_poly_eq()},
+#' \code{stat_quant_line()}, \code{stat_quant_eq()}, \code{stat_ma_line()},
+#' \code{stat_ma_eq()}.
 #'
-#' \code{\link[ggplot2]{geom_smooth}}, which is used by default, treats each
-#' axis differently and can thus have two orientations. The orientation is easy
-#' to deduce from the argument passed to \code{formula}. Thus,
-#' \code{stat_poly_line()} will by default guess which orientation the layer
-#' should have. If no argument is passed to \code{formula}, the orientation is
-#' ambiguous. In that case the orientation can be specified directly passing an
-#' argument to the \code{orientation} parameter, which can be either \code{"x"}
-#' or \code{"y"}. The value gives the axis that is taken as the explanatory
-#' variable, \code{"x"} being the default orientation you would expect for the
-#' geom. Package 'ggpmisc' does not define new geometries matching the new
-#' statistics as they are not needed and conceptually transformations of
-#' \code{data} are expressed as statistics.
+#' \code{geom_poly_line()} treats the x and y aesthetics differently and can
+#' thus have two orientations. The orientation can be deduced from the argument
+#' passed to \code{formula}. Thus, \code{stat_poly_line()} will by default guess
+#' which orientation the layer should have. If no argument is passed to
+#' \code{formula}, the formula defaults to \code{y ~ x}. For consistency with
+#' \code{\link[ggplot2]{stat_smooth}} orientation can be also specified directly
+#' passing an argument to the \code{orientation} parameter, which can be either
+#' \code{"x"} or \code{"y"}. The value of \code{orientation} gives the axis that
+#' is taken as the explanatory variable. Package 'ggpmisc' does not define
+#' new geometries matching the new statistics as they are not needed and
+#' conceptually transformations of \code{data} are statistics in the grammar
+#' of graphics.
 #'
 #' @param mapping The aesthetic mapping, usually constructed with
 #'   \code{\link[ggplot2]{aes}} or \code{\link[ggplot2]{aes_}}. Only needs to be
@@ -60,20 +55,18 @@
 #' @param method.args named list with additional arguments.
 #' @param se Display confidence interval around smooth? (`TRUE` by default, see
 #'   `level` to control.)
+#' @param mf.values logical Add R2, adjusted R2, p-value and n as columns to
+#'   returned data? (`FALSE` by default.)
 #' @param fullrange Should the fit span the full range of the plot, or just
 #'   the data?
 #' @param level Level of confidence interval to use (0.95 by default).
-#' @param span Controls the amount of smoothing for the default loess smoother.
-#'   Smaller numbers produce wigglier lines, larger numbers produce smoother
-#'   lines. Only used with loess, i.e. when `method = "loess"`,
-#'   or when `method = NULL` (the default) and there are fewer than 1,000
-#'   observations.
 #' @param n Number of points at which to evaluate smoother.
 #' @param orientation character Either "x" or "y" controlling the default for
 #'   \code{formula}.
 #'
 #' @return The value returned by the statistic is a data frame, that will have
-#'   \code{n} rows of predicted values and and their confidence limits.
+#'   \code{n} rows of predicted values and their confidence limits. Optionally
+#'   it will also include additional values related to the model fit.
 #'
 #' @section Computed variables: `stat_poly_line()` provides the following
 #'   variables, some of which depend on the orientation: \describe{ \item{y *or*
@@ -81,6 +74,13 @@
 #'   interval around the mean} \item{ymax *or* xmax}{upper pointwise confidence
 #'   interval around the mean} \item{se}{standard error} }
 #'
+#'   If \code{mf.values = TRUE} is passed then columns based on the summary of
+#'   the model fit are added, with the same value in each row within a group.
+#'   This is wasteful and disabled by default, but provides a simple and robust
+#'   approach to achieve effects like colouring or hiding of the model fit line
+#'   based on P-values, r-squared, adjusted r-squared or the number of
+#'   observations.
+#'
 #' @section Aesthetics: \code{stat_poly_line} understands \code{x} and \code{y},
 #'   to be referenced in the \code{formula} and \code{weight} passed as argument
 #'   to parameter \code{weights}. All three must be mapped to \code{numeric}
@@ -109,26 +109,6 @@
 #'   geom_point() +
 #'   stat_poly_line(formula = x ~ poly(y, 3))
 #'
-#' # The default behavior of geom_smooth()
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   stat_poly_line(method = "auto")
-#'
-#' # Use span to control the "wiggliness" of the default loess smoother.
-#' # The span is the fraction of points used to fit each local regression:
-#' # small numbers make a wigglier curve, larger numbers make a smoother curve.
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   stat_poly_line(method = "loess", span = 0.3)
-#'
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   stat_poly_line(method = lm, formula = y ~ splines::bs(x, 3), se = FALSE)
-#'
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   stat_poly_line(method = lm, formula = x ~ splines::bs(y, 3), se = FALSE)
-#'
 #' # Smooths are automatically fit to each group (defined by categorical
 #' # aesthetics or the group aesthetic) and for each facet.
 #'
@@ -138,9 +118,24 @@
 #'
 #' ggplot(mpg, aes(displ, hwy)) +
 #'   geom_point() +
-#'   stat_poly_line(method = "auto", span = 0.8) +
+#'   stat_poly_line() +
 #'   facet_wrap(~drv)
 #'
+#' # Inspecting the returned data using geom_debug()
+#' if (requireNamespace("gginnards", quietly = TRUE)) {
+#'   library(gginnards)
+#'
+#'   ggplot(mpg, aes(displ, hwy)) +
+#'     stat_poly_line(geom = "debug")
+#'
+#'   ggplot(mpg, aes(displ, hwy)) +
+#'     stat_poly_line(geom = "debug", mf.values = TRUE)
+#'
+#'   ggplot(mpg, aes(displ, hwy)) +
+#'     stat_poly_line(geom = "debug", method = lm, mf.values = TRUE)
+#'
+#' }
+#'
 #' @export
 #'
 stat_poly_line <- function(mapping = NULL, data = NULL,
@@ -149,8 +144,8 @@ stat_poly_line <- function(mapping = NULL, data = NULL,
                            method = "lm",
                            formula = NULL,
                            se = TRUE,
+                           mf.values = FALSE,
                            n = 80,
-                           span = 0.75,
                            fullrange = FALSE,
                            level = 0.95,
                            method.args = list(),
@@ -179,39 +174,159 @@ stat_poly_line <- function(mapping = NULL, data = NULL,
     }
   }
 
+  # here so that warnings are triggered before rendering
   if (is.character(method)) {
+    method.name <- method
     if (method == "rlm") {
       method <- MASS::rlm
     } else if (method == "rq") {
       warning("Method 'rq' not supported, please use 'stat_quant_line()'.")
-      method <- "auto"
+      method <- "lm"
+    } else if (method == "auto") {
+      message("Method 'auto' is equivalent to 'lm', please use 'stat_smooth()' for splines.")
+      method <- "lm"
     }
+  } else {
+    method.name <- "function"
   }
 
   ggplot2::layer(
     data = data,
     mapping = mapping,
-    stat = ggplot2::StatSmooth,
+    stat = StatPolyLine,
     geom = geom,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
     params = list(
       method = method,
+      method.name = method.name,
       formula = formula,
       se = se,
+      mf.values = mf.values,
       n = n,
       fullrange = fullrange,
       level = level,
       na.rm = na.rm,
       orientation = orientation,
       method.args = method.args,
-      span = span,
       ...
     )
   )
 }
 
+poly_line_compute_group_fun <-
+  function(data, scales,
+           method = NULL,
+           method.name = NA_character_,
+           formula = NULL,
+           se = TRUE,
+           mf.values = FALSE,
+           n = 80,
+           fullrange = FALSE,
+           xseq = NULL,
+           level = 0.95,
+           method.args = list(),
+           na.rm = FALSE,
+           flipped_aes = NA,
+           orientation = "x") {
+    data <- ggplot2::flip_data(data, flipped_aes)
+    if (length(unique(data$x)) < 2) {
+      # Not enough data to perform fit
+      return(data.frame())
+    }
+
+    if (is.null(data$weight)) data$weight <- 1
+
+    if (is.null(xseq)) {
+      if (fullrange) {
+        xrange <- scales[[orientation]]$dimension()
+      } else {
+        xrange <- range(data$x, na.rm = TRUE)
+      }
+      xseq <- seq(from = xrange[1], to = xrange[2], length.out = n)
+    }
+
+    if (is.character(method)) {
+      method <- match.fun(method)
+    }
+
+    base.args <- list(quote(formula),
+                      data = quote(data),
+                      weights = quote(weight))
+    mf <- do.call(method, c(base.args, method.args))
+
+    newdata <- data.frame(x = xseq)
+    prediction <- stats::predict(mf,
+                                 newdata = newdata,
+                                 se.fit = se,
+                                 level = level,
+                                 interval = if (se) "confidence" else "none"
+    )
+    if (se) {
+      if (exists("fit", prediction)) {
+        prediction <- as.data.frame(prediction[["fit"]])
+      }
+      names(prediction) <- c("y", "ymin", "ymax")
+    } else {
+      prediction <- data.frame(y = as.vector(prediction))
+    }
+    prediction <- cbind(newdata, prediction)
+
+    if (mf.values) {
+      mf.summary <- summary(mf)
+      # summary methods for different fit methods may no return some slots
+      if (exists("fstatistic", mf.summary)) {
+        prediction[["p.value"]] <-
+          stats::pf(q = mf.summary[["fstatistic"]][["value"]],
+                    df1 = mf.summary[["fstatistic"]][["numdf"]],
+                    df2 = mf.summary[["fstatistic"]][["dendf"]],
+                    lower.tail = FALSE)
+      } else {
+        prediction[["p.value"]] <- NA_real_
+      }
+      if (exists("r.squared", mf.summary)) {
+        prediction[["r.squared"]] <- mf.summary[["r.squared"]]
+      } else {
+        prediction[["r.squared"]] <- NA_real_
+      }
+      if (exists("adj.r.squared", mf.summary)) {
+        prediction[["adj.r.squared"]] <- mf.summary[["adj.r.squared"]]
+      } else {
+        prediction[["adj.r.squared"]] <- NA_real_
+      }
+      if (exists("residuals", mf)) {
+        prediction[["n"]] <- length(mf[["residuals"]])
+      } else {
+        prediction[["n"]] <- NA_real_
+      }
+      prediction[["method"]] <- method.name
+    }
+
+    prediction$flipped_aes <- flipped_aes
+    ggplot2::flip_data(prediction, flipped_aes)
+  }
+
+#' @rdname ggpmisc-ggproto
+#' @format NULL
+#' @usage NULL
+#' @export
+StatPolyLine <-
+  ggplot2::ggproto("StatPolyLine", Stat,
+                   setup_params = function(data, params) {
+                     params[["flipped_aes"]] <-
+                       ggplot2::has_flipped_aes(data, params, ambiguous = TRUE)
+                     params
+                   },
+
+                   extra_params = c("na.rm", "orientation"),
+
+                   compute_group = poly_line_compute_group_fun,
+
+                   required_aes = c("x", "y")
+  )
+
+
 #' Swap x and y in a formula
 #'
 #' By default a formula of x on y is converted into a formula of y
